import 'package:flutter_test/flutter_test.dart';
import 'package:flutter_extension_kits/flutter_extension_kits.dart';

void main() {
  test('format String', () {
    // Given
    // const String text = 'Today is %2\$ and tomorrow is %1\$';
    String text = 'Today is {1} and tomorrow is {0}';
    final List<String> placeHolders = List<String>()..add('Tuesday')..add('Monday')..add('extra');
    String expected = 'Today is Monday and tomorrow is Tuesday';

    // When
    String actual = text.format(placeHolders);

    // Then
    print(actual);
    expect(actual, expected);

    //Testing empty placeholders
    text = 'Today is {1} and tomorrow is {0}';
    List<String> placeHoldersEmpty = List<String>();
    expected = 'Today is {1} and tomorrow is {0}';

    // When
    actual = text.format(placeHoldersEmpty);

    print(actual);
    expect(actual, expected);
  });

  final dateA = DateTime(2020, 10, 7, 20, 10, 30);
  final dateB = DateTime(2020, 8, 2, 10, 5, 10);
  final dateC = DateTime(2020, 10, 7, 20, 10, 30);

  // dateA - dateB
  print(dateA.diffDays(dateB)); // 66
  print(dateA.diffHours(dateB)); // 1594
  print(dateA.diffMinutes(dateB)); // 95645
  print(dateA.diffSeconds(dateB)); // 5738720

  //String Format
  String txtFormat = 'Second params {1} and first is {0}';
  final List<String> testParams = List<String>()..add('Tuesday')..add('Monday');
  print(txtFormat.format(testParams));

  //Time Left
  // Current testing time is 2020/05/25 9:11PM
  final expiryDate01 = DateTime(2021, 10, 7, 20, 10, 30);
  print(expiryDate01.beautifyTime(strSuffix: 'left')); //1 year 5 months left

  final expiryDate02 = DateTime(2020, 10, 7, 20, 10, 30);
  print(expiryDate02.beautifyTime(strSuffix: 'left')); //4 months left

  final expiryDate03 = DateTime(2020, 5, 15, 20, 10, 30);
  print(expiryDate03.beautifyTime(strSuffix: 'left')); //10 days left

  final expiryDate04 = DateTime(2020, 5, 25, 20, 23, 30);
  print(expiryDate04.beautifyTime(strSuffix: 'left')); //46 minutes left

  // Current testing time is 2020/05/25 8:53PM
  final lastUpdatedDate01 = DateTime(2018, 12, 7, 20, 5, 10);
  print(lastUpdatedDate01.beautifyTime(bUTC: false, bAddPlural: true)); //1year 6months ago

  final lastUpdatedDate02 = DateTime(2019, 10, 7, 20, 5, 10);
  print(lastUpdatedDate02.beautifyTime(bUTC: false, bAddPlural: true)); //8months ago

  final lastUpdatedDate03 = DateTime(2020, 5, 25, 10, 5, 10);
  print(lastUpdatedDate03.beautifyTime(bUTC: false, bAddPlural: true)); //10hours ago

  final lastUpdatedDate04 = DateTime(2020, 5, 25, 20, 5, 10);
  print(lastUpdatedDate04.beautifyTime(bUTC: false, bAddPlural: true)); //48minutes ago

  final lastUpdatedDate05 = DateTime(2020, 5, 25, 20, 53, 10);
  print(lastUpdatedDate05.beautifyTime(bUTC: false, bAddPlural: true)); //Just now

  //Testing events
  var someEvent = ExtendedEvent();
  someEvent.subscribe('key01', (args) {
    print('Event triggered');
  });

  print('Triggering event');
  someEvent.broadcast();

  print('Total subscribers : ' + someEvent.subscriberCount.toString());

  bool bUnsubscribeStatus = someEvent.unsubscribe('key02');
  if (!bUnsubscribeStatus)
    print('Failed to unsubscribe event');
  else
    print('key02 unsubscribed from the manager');

  bUnsubscribeStatus = someEvent.unsubscribe('key01');
  if (!bUnsubscribeStatus)
    print('Failed to unsubscribe event');
  else
    print('key01 unsubscribed from the manager');

  //Trigger won't reived anymore since we've unsubscribed from the manager
  someEvent.broadcast();
  print('Total subscribers : ' + someEvent.subscriberCount.toString());
}
